<?php

Yii::import('system.console.CConsoleCommand');

class AuthInitCommand extends CConsoleCommand {

  const OPERATION = 0;
  const TASK      = 1;
  const ROLE      = 2;

  const DEFAULT_CONFIG = "authinit.php";

  private $auth;

  public function run($args)
  {
    if(!isset($args[0]))
      $file = Yii::getPathOfAlias('application.config').'/'.self::DEFAULT_CONFIG;
    else
      $file = $args[0];
    
    if(!is_file($file))
      $this->usageError("the configuration file {$file} does not exist.");

    $config = include($file);

    $this->auth=Yii::app()->authManager;

    // TODO: provide a mechanism to preserve assignments
    $this->auth->clearAll();
    
    if(isset($config['operations']) && is_array($config['operations'])){
      $operations = $config['operations'];
      foreach($operations as $l => $r){
	$operation = $l;
	$options = $r;
	if(is_int($l)){
	  $operation = $r;
	  $options = array();
	}

	$desc = $options['description'] ? $options['description'] : '';
	$biz  = $options['bizrule'    ] ? $options['bizrule'    ] : '';
	$data = $options['data'       ] ? $options['data'       ] : '';

	$this->auth->createAuthItem($operation, self::OPERATION, $desc, $biz, $data);
	
	$this->addChildOperations($operation, $options['operations']);
      }
    }
    
    if(isset($config['tasks']) && is_array($config['tasks'])){
      $tasks = $config['tasks'];
      foreach($tasks as $l => $r){
	$task = $l;
	$options = $r;
	if(is_int($l)){
	  $task = $r;
	  $options = array();
	}
	
	$desc = $options['description'] ? $options['description'] : '';
	$biz  = $options['bizrule'    ] ? $options['bizrule'    ] : '';
	$data = $options['data'       ] ? $options['data'       ] : '';
	
	$this->auth->createAuthItem($task, self::TASK, $desc, $biz, $data);

	$this->addChildOperations($task, $options['operations']);
	$this->addChildTasks($task, $options['tasks']);

      }
    }
    
    if(isset($config['roles']) && is_array($config['roles'])){
      $roles = $config['roles'];
      foreach($roles as $l => $r){
	$role = $l;
	$options = $r;
	if(is_int($l)){
	  $role = $r;
	  $options = array();
	}
	
	$desc = $options['description'] ? $options['description'] : '';
	$biz  = $options['bizrule'    ] ? $options['bizrule'    ] : '';
	$data = $options['data'       ] ? $options['data'       ] : '';
	
	$this->auth->createAuthItem($role, self::ROLE, $desc, $biz, $data);

	$this->addChildOperations($role, $options['operations']);
	$this->addChildTasks($role, $options['tasks']);
	$this->addChildRoles($role, $options['roles']);

      }
    }
    
    if(isset($config['assignments']) && is_array($config['assignments'])){
      foreach($config['assignments'] as $user => $item){
	$biz = null;
	$data = null;
	if(is_array($item)){
	  $item = $item['item'   ];
	  $biz  = $item['bizrule'] ? $item['bizrule'] : null;
	  $data = $item['data'   ] ? $item['data'   ] : null;
	}
	if(!$this->auth->isAssigned($item, $user)){
	  $this->auth->assign($item, $user, $biz, $data);
	}
      }
    }
    
  }

  private function addChildOperations($parent, $operations)
  {
    if($operations && is_array($operations)){
      foreach($operations as $idx => $operation){
	if(!($op = $this->auth->getAuthItem($operation))){
	  $this->auth->createAuthItem($operation, self::OPERATION);
	}
	$this->auth->addItemChild($parent, $operation);
      }
    }    
  }

  private function addChildTasks($parent, $tasks)
  {
    if($tasks && is_array($tasks)){
      foreach($tasks as $idx => $task){
	$this->auth->addItemChild($parent, $task);
      }
    }
  }
  
  private function addChildRoles($parent, $roles)
  {
    if($roles && is_array($roles)){
      foreach($roles as $idx => $role){
	$this->auth->addItemChild($parent, $role);
      }
    }
  }
  
    
}
